import logging
from django.conf import settings
from django.core.management.base import CommandError
from bakery.management.commands import BasePublishCommand
logger = logging.getLogger(__name__)


class Command(BasePublishCommand):
    help = "Empties the Amazon S3 bucket defined in settings.py"
    bucket_unconfig_msg = "Bucket unconfigured. Set AWS_BUCKET_NAME in settings.py or provide it with --aws-bucket-name"

    def add_arguments(self, parser):
        parser.add_argument(
            "--aws-bucket-name",
            action="store",
            dest="aws_bucket_name",
            default='',
            help="Specify the AWS bucket to sync with. Will use settings.AWS_BUCKET_NAME by default."
        )

    def handle(self, *args, **options):
        if options.get("aws_bucket_name"):
            aws_bucket_name = options.get("aws_bucket_name")
        else:
            # Otherwise try to find it the settings
            if not hasattr(settings, 'AWS_BUCKET_NAME'):
                raise CommandError(self.bucket_unconfig_msg)
            aws_bucket_name = settings.AWS_BUCKET_NAME

        # Pull all the keys from the bucket
        all_objects = self.get_all_objects_in_bucket(aws_bucket_name)
        keys = all_objects.keys()
        self.batch_delete_s3_objects(keys, aws_bucket_name)

        # A little logging
        logger.info("unpublish completed, %d deleted files" % len(keys))



import logging
from django.conf import settings
from django.core import management
from django.contrib.contenttypes.models import ContentType
logger = logging.getLogger(__name__)
try:
    from celery import shared_task
except ImportError:
    raise ImportError("celery must be installed to use django-bakery's tasks")


@shared_task()
def publish_object(content_type_pk, obj_pk):
    """
    Build all views related to an object, and then sync with S3.

    Accepts primary keys to retrieve a model object that
    inherits bakery's BuildableModel class.
    """
    ct = ContentType.objects.get_for_id(content_type_pk)
    obj = ct.get_object_for_this_type(pk=obj_pk)
    try:
        # Build the object
        logger.info("publish_object task has received %s" % obj)
        obj.build()
        # Run the `publish` management command unless the
        # ALLOW_BAKERY_AUTO_PUBLISHING variable is explictly set to False.
        if not getattr(settings, 'ALLOW_BAKERY_AUTO_PUBLISHING', True):
            logger.info("Not running publish command because \
ALLOW_BAKERY_AUTO_PUBLISHING is False")
        else:
            management.call_command("publish")
    except Exception:
        # Log the error if this crashes
        logger.error("Task Error: publish_object", exc_info=True)


@shared_task()
def unpublish_object(content_type_pk, obj_pk):
    """
    Unbuild all views related to a object and then sync to S3.

    Accepts primary keys to retrieve a model object that
    inherits bakery's BuildableModel class.
    """
    ct = ContentType.objects.get_for_id(content_type_pk)
    obj = ct.get_object_for_this_type(pk=obj_pk)
    try:
        # Unbuild the object
        logger.info("unpublish_object task has received %s" % obj)
        obj.unbuild()
        # Run the `publish` management command unless the
        # ALLOW_BAKERY_AUTO_PUBLISHING variable is explictly set to False.
        if not getattr(settings, 'ALLOW_BAKERY_AUTO_PUBLISHING', True):
            logger.info("Not running publish command because \
ALLOW_BAKERY_AUTO_PUBLISHING is False")
        else:
            management.call_command("publish")
    except Exception:
        # Log the error if this crashes
        logger.error("Task Error: unpublish_object", exc_info=True)
